#! /usr/bin/perl
#
# Copyright (c) 2011 by Evangelos A. Coutsias and Michael J. Wester
# Department of Mathematics and Statistics
# University of New Mexico, Albuquerque, New Mexico, USA
# Written by Michael J. Wester
#
$PLOP_data_dir = "/users/local/src/plop/data";
$tolerance = 0.001;
$max_iterations = 1000;
$CONTROL_file = "tmp.con";
#
if (@ARGV < 1)
   {
   print "Usage: PLOP_minimize.pl <PDB_basename> {C<mol_chain>} ",
         "{<start_res#-end_res#>} ...\n";
   exit(1);
   }
#
($PDB_basename = $ARGV[0]) =~ s/.pdb$//;
#
if (! -r "$PDB_basename.pdb")
   {
   print "$PDB_basename.pdb cannot be read!\n";
   exit(2);
   }
#
# If this (a two character) argument is provided, the molecule is assumed
# cyclic and so loaded a bit differently.
#
if (@ARGV >= 2 && length($ARGV[1]) <= 2)
   {
   $mol_chain = substr($ARGV[1], 1, 1);
   shift(@ARGV);
   }
#
$n_loops = @ARGV - 1;
for ($i = 0; $i < $n_loops; ++$i)
   {
   ($line = $ARGV[$i + 1]) =~ s/^([^-]+)-([^-]+)$/(\1, \2)/;
   ($loop[$i][0], $loop[$i][1]) = eval($line);
   }
#
open(CONTROL, "> $CONTROL_file");
print CONTROL "datadirectory $PLOP_data_dir\n";
print CONTROL "outputdirectory .\n";
print CONTROL "jobname $PDB_basename\n";
#print CONTROL "log tmp.log\n";
if (defined($mol_chain))
   {
   print CONTROL "load pdb $PDB_basename.pdb cyclic $mol_chain\n";
   }
else
   {
   print CONTROL "load pdb $PDB_basename.pdb\n";
   }
print CONTROL "minimize &\n";
if ($n_loops == 0)
   {
   print CONTROL "   residues all residues &\n";
   }
else
   {
   for ($i = 0; $i < $n_loops; ++$i)
      {
      print CONTROL "   residues _:$loop[$i][0] _:$loop[$i][1] &\n";
      }
   }
print CONTROL "   rmsgradient $tolerance &\n";
print CONTROL "   mxitn $max_iterations\n";
print CONTROL "energy calculate\n";
print CONTROL "write pdb m_$PDB_basename.pdb\n";
close(CONTROL);
#
open(PLOP, "echo $CONTROL_file | plop |");
$printing = 0;
while ($line = <PLOP>)
   {
   chomp($line);
   if ($line =~ /^\s*$/)
      {
      $printing = 0;
      }
   if ($printing)
      {
      print STDERR "plop: $line\n";
      }
   if ($line =~ /ERROR/ || $line =~ /no match in memory/)
      {
      $printing = 1;
      print STDERR "plop: $line\n";
      }
   elsif ($line =~ /TOTALE/)
      {
      ($E = $line) =~ s/^\s*TOTALE\s+(\S+)\s*$/\1/;
      }
   }
close(PLOP);
#
if (defined($E))
   {
   print "$E\n";
   }
else
   {
   print STDERR "plop terminated abnormally.\n";
   exit(3);
   }
#
#unlink("$CONTROL_file");
